{-------------------------------------------------------------------------
	Module:		TGlobe Thematic mapping Demo program

	Comment:	Simple thematic mapping demo program for TGlobe.

	Author:		Graham Knight
	Email:		gknight@helmstone.co.uk
	Version:	1.0
	Date:			February 1999

	Creates a TThematicPolygonPresenter to display thematic polygons.

	Uses the TThematic object to calculate the Break limits for the display
	of the Thematic information.
-------------------------------------------------------------------------}
unit UThema;

interface

uses
	WinProcs, WinTypes, Messages, SysUtils, Classes, Graphics, Controls, Forms,
	Dialogs, StdCtrls, Buttons, Spin, ExtCtrls, Menus, FileCtrl, DB,
	DBTables, Globe3, TGThematic;

type
	{----------------------------------------------------
		This is an example of how to create a Thematic Presenter.
		This uses the Thematic object from TGThematic to
		calculate the statistics and Break limits from the objects
		in the Presenters Layer.
	----------------------------------------------------}
	TThematicPolygonPresenter = class( TPolygonPresenter )
	protected
		Thematic : TGlobeThematic;
	public
		constructor CreateObject( ParentLayer : TGlobeLayer ); override;
		destructor Destroy; override;
		procedure RenderObject( Geod : TGlobeObject ); override;
	end;

	{--------------------------------------------------------------------}
	TfrmMain = class(TForm)
		ColorDialog1: TColorDialog;
		Panel2: TPanel;
		pnlMouse: TPanel;
		Globe: TGlobe3;
		MainMenu1: TMainMenu;
		File1: TMenuItem;
		Exit1: TMenuItem;
		About1: TMenuItem;
		Panel1: TPanel;
		Table1: TTable;
		Panel3: TPanel;
		Label3: TLabel;
		Label8: TLabel;
		lbxColumns: TListBox;
		GroupBox1: TGroupBox;
    lblRange: TLabel;
		Label2: TLabel;
    Label1: TLabel;
    lblMean: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    spnGroups: TSpinEdit;
    edtNoData: TEdit;
    pnlNoData: TPanel;
    pnlStart: TPanel;
    pnlEnd: TPanel;
    Panel4: TPanel;
		lbxGroups: TListBox;
    pnlZoom: TPanel;
    btnZoomIn: TBitBtn;
    btnZoomOut: TBitBtn;
    btnZoomExtents: TBitBtn;
    procedure btnZoomInClick(Sender: TObject);
    procedure btnZoomOutClick(Sender: TObject);
		procedure btnZoomExtentsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
		procedure PanelColorClick(Sender: TObject);
		procedure lbxColumnsClick(Sender: TObject);
		procedure GlobeMouseMove(Sender: TObject; Shift: TShiftState; X,
			Y: Integer);
		procedure edt1Exit(Sender: TObject);
		procedure edt1Enter(Sender: TObject);
		procedure lbxGroupsDrawItem(CONTROL: TWinControl; index: Integer;
			Rect: TRect; State: TOwnerDrawState);
		procedure About1Click(Sender: TObject);
		procedure Exit1Click(Sender: TObject);
		procedure spnGroupsChange(Sender: TObject);
		procedure lbxGroupsClick(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		ThematicPresenter : TThematicPolygonPresenter;
		WorkIngLayer : TGlobeLayer;
	end;

var
	frmMain       : TfrmMain;
	gEditValue    : Double;

implementation

{$R *.DFM}

{------------------------------------------------------------------------------
	TThematicPolygonPresenter.CreateObject
------------------------------------------------------------------------------}
constructor TThematicPolygonPresenter.CreateObject( ParentLayer : TGlobeLayer );
begin
	inherited CreateObject( ParentLayer );
	Thematic := TGlobeThematic.Create( Self );
end;

{------------------------------------------------------------------------------
	TThematicPolygonPresenter.Destroy
------------------------------------------------------------------------------}
destructor TThematicPolygonPresenter.Destroy;
begin
	Thematic.Free;
	inherited Destroy;
end;

{------------------------------------------------------------------------------
	TThematicPolygonPresenter.RenderObject
------------------------------------------------------------------------------}
procedure TThematicPolygonPresenter.RenderObject( Geod : TGlobeObject );
var
	ABreak : TThematicBreak;
begin
	{ Get the break associated with the objects value }
	ABreak := Thematic.ValueToBreak( Geod.Value );

	if ABreak <> nil then
	begin
		PolygonBrush.BrushColor := ABreak.Color;
		PolygonBrush.RenderAttribute( ABreak.Selected );

		PolygonPen.PenColor := ABreak.Color;
		PolygonPen.RenderAttribute( ABreak.Selected );
	end;

	Globe.DrawLLPolygon( Geod );	{ actually paint the polygon }
end;




{------------------------------------------------------------------------------
	TfrmMain.FormCreate
------------------------------------------------------------------------------}
procedure TfrmMain.FormCreate(Sender: TObject);
var
	Obj : TGlobeObject;
	idx : integer;
begin
	{ load the background data }
	Globe.ProfileName := 'TGlobe.prf';

	{ Create a working layer and the Thematic Polygon Presenter }
	WorkIngLayer := Globe.LayerNew( 'ThematicLayer' );
	ThematicPresenter := TThematicPolygonPresenter.Create( WorkIngLayer, 'ThematicPolygon', 1 );

	{ set up the display panel from the Presenter }
	with ThematicPresenter do
	begin
		pnlStart.Color := Thematic.StartColor;
		pnlEnd.Color := Thematic.EndColor;
		pnlNoData.Color := Thematic.NoDataColor;
	end;

	{ Load the States data }
	WorkIngLayer.LoadFromFile('.\States.lyr');

	{ Read the States DBF file and get a list of Numeric fields }
	with Table1 do
	begin
		TableName := 'STATES.DBF';
		Active := True;

		{ build list of Numeric fields }
		lbxColumns.Items.Clear;
		for idx := 0 to FieldCount - 1 do
			if Fields[idx].DataType = ftFloat then
				lbxColumns.Items.AddObject( Fields[idx].FieldName, TObject(idx));
	end;

	{ Zoom and center the globe }
	Globe.Altitude := 0;
end;

{------------------------------------------------------------------------------
	TfrmMain.Exit1Click
------------------------------------------------------------------------------}
procedure TfrmMain.Exit1Click(Sender: TObject);
begin
	Close;
end;

{------------------------------------------------------------------------------
	TfrmMain.About1Click
------------------------------------------------------------------------------}
procedure TfrmMain.About1Click(Sender: TObject);
begin
	ShowMessage('TGlobe Simple Thematic Demo Program'
		 + #10#10'Copyright (c) Graham Knight 1997 - 1999'
		 + #10#10'Email: gknight@helmstone.co.uk'
		 + #10'http://www3.mistral.co.uk/helmstone/tglobe');
end;

{------------------------------------------------------------------------------
	TfrmMain.btnZoomInClick
------------------------------------------------------------------------------}
procedure TfrmMain.btnZoomInClick(Sender: TObject);
begin
	with Globe do
		ScaleFactor := ScaleFactor * 2;
end;

{------------------------------------------------------------------------------
	TfrmMain.btnZoomOutClick
------------------------------------------------------------------------------}
procedure TfrmMain.btnZoomOutClick(Sender: TObject);
begin
	with Globe do
		ScaleFactor := ScaleFactor / 2;
end;

{------------------------------------------------------------------------------
	TfrmMain.btnZoomExtentsClick
------------------------------------------------------------------------------}
procedure TfrmMain.btnZoomExtentsClick(Sender: TObject);
begin
	Globe.Altitude := 0;
end;

{------------------------------------------------------------------------------
	TfrmMain.GlobeMouseMove
------------------------------------------------------------------------------}
procedure TfrmMain.GlobeMouseMove(Sender: TObject; Shift: TShiftState; X,
		Y: Integer);
var
	Obj : TGlobeObject;
begin
	{ display information about the object under the mouse }
	Obj := Globe.ObjectAtXY(X, Y);
	if Obj <> nil then
		pnlMouse.Caption := Format('%s  %f',[Obj.Title, Obj.Value])
	else
		pnlMouse.Caption := '';
end;

{------------------------------------------------------------------------------
	TfrmMain.edt1Enter
------------------------------------------------------------------------------}
procedure TfrmMain.edt1Enter(Sender: TObject);
begin
	gEditValue := StrToFloat(TEdit(Sender).Text);
end;

{------------------------------------------------------------------------------
	TfrmMain.edt1Exit
------------------------------------------------------------------------------}
procedure TfrmMain.edt1Exit(Sender: TObject);
var
	eTmp: Double;
begin
	with TEdit(Sender) do
		try
			eTmp := StrToFloat(Text);
			gEditValue := eTmp;
		except { swallow conversion exception }
		end;
	Text := FloatToStr(gEditValue);
end;

{------------------------------------------------------------------------------
	TfrmMain.lbxGroupsDrawItem
------------------------------------------------------------------------------}
procedure TfrmMain.lbxGroupsDrawItem(CONTROL: TWinControl; index: Integer;
		Rect: TRect; State: TOwnerDrawState);
begin
	with lbxGroups do
	begin
		Canvas.TextRect(Rect, 2, Rect.Top, Items[index]);
		Rect.Left := (Width div 4) * 3;
		InflateRect(Rect, -1, -1);
		Canvas.Brush.Color := ThematicPresenter.Thematic.Breaks[index].Color;
		Canvas.FillRect(Rect);
	end;
end;

{------------------------------------------------------------------------------
	TfrmMain.lbxGroupsClick
------------------------------------------------------------------------------}
procedure TfrmMain.lbxGroupsClick(Sender: TObject);
begin
	ThematicPresenter.Thematic.SelectBreak( lbxGroups.ItemIndex );
	Globe.Redraw;
end;

{------------------------------------------------------------------------------
	TfrmMain.lbxColumnsClick
------------------------------------------------------------------------------}
procedure TfrmMain.lbxColumnsClick(Sender: TObject);
var
	sTitle : string;
	iDataIndex : integer;
	Obj : TGlobeObject;
begin
	with ThematicPresenter do
	begin
		{ get the no data value if present }
		if edtNoData.Text <> '' then
			Thematic.NoDataValue := StrToFloat(edtNoData.Text)
		else
			Thematic.NoDataValue := -99.0;

		{ get the column index for the value column }
		with lbxColumns do
			iDataIndex := Integer(Items.Objects[ItemIndex]);

		{ scan the layer and update the values in the objects from the database }
		Table1.First;
		while not Table1.EOF do
		begin
			sTitle := Table1.FieldValues['STATE_NAME'];
			Obj := WorkIngLayer.ObjectByTitle( sTitle, nil );
			while Obj <> nil do
			begin
				Obj.Presenter := ThematicPresenter;
				Obj.Value := Table1.Fields[iDataIndex].AsFloat;
				{ find all occourances of this object }
				Obj := WorkIngLayer.ObjectByTitle( sTitle, Obj );
			end;
			Table1.Next;
		end;

		{ set up the thematic object}
		with Thematic do
		begin
			BreakCount := spnGroups.Value;	{ set the number of breaks }
			CalcStatistics;		{ Get min, Max, Mean }
			CalcBreakLimits( btPercentile );	{ Calculate breaks on a Precentile basis }
			GradientColors;	{ set the break colors to a gradient }

			lblRange.Caption := Format('Range: %0.2f to %0.2f',[MinValue, MaxValue]);
			lblMean.Caption := Format('Mean: %0.2f',[MeanValue]);
		end;
		spnGroupsChange(nil);
	end;

	Globe.Redraw;
end;

{------------------------------------------------------------------------------
	TfrmMain.spnGroupsChange
------------------------------------------------------------------------------}
procedure TfrmMain.spnGroupsChange(Sender: TObject);
var
	idx: Integer;
begin
	with ThematicPresenter, Thematic do
	try
		BreakCount := spnGroups.Value;
		CalcStatistics;		{ Get min, Max, Mean }
		CalcBreakLimits( btPercentile );	{ Calculate breaks on a Precentile basis }
		GradientColors;	{ set the break colors to a gradient }

		lbxGroups.Items.Clear;
		lbxGroups.Items.Add( Format('0.0  - %0.2f',[Breaks[0].Limit]));
		for idx := 1 to Thematic.BreakCount - 1 do
			lbxGroups.Items.Add( Format('%0.2f - %0.2f',[Breaks[idx - 1].Limit, Breaks[idx].Limit]));

		Globe.Redraw;
	except
	end;
end;

{------------------------------------------------------------------------------
	TfrmMain.PanelColorClick
------------------------------------------------------------------------------}
procedure TfrmMain.PanelColorClick(Sender: TObject);
begin
	ColorDialog1.Color := TPanel(Sender).Color;
	if ColorDialog1.Execute then
	begin
		TPanel(Sender).Color := ColorDialog1.Color;

		with ThematicPresenter.Thematic do
		begin
			if Sender = pnlNoData then
				NoDataColor := ColorDialog1.Color;
			if Sender = pnlStart then
				StartColor := ColorDialog1.Color;
			if Sender = pnlEnd then
				EndColor := ColorDialog1.Color;
		end;

		lbxColumnsClick( nil );
		lbxGroups.Invalidate;
		Globe.Redraw;
	end;
end;

end.

